@@ -39,11 +39,11 @@ class ViewDefinition(object):
     even exist yet. That can be fixed using the `sync` method:
 
     >>> view.sync(db)                                       #doctest: +ELLIPSIS
-    [(True, '_design/tests', ...)]
+    [(True, u'_design/tests', ...)]
     >>> design_doc = view.get_doc(db)
     >>> design_doc                                          #doctest: +ELLIPSIS
-    <Document '_design/tests'@'...' {...}>
-    >>> print design_doc['views']['all']['map']
+    <Document u'_design/tests'@u'...' {...}>
+    >>> print(design_doc['views']['all']['map'])
     function(doc) {
         emit(doc._id, null);
     }
@@ -55,11 +55,11 @@ class ViewDefinition(object):
     ...     yield doc['somekey'], doc['somevalue']
     >>> view = ViewDefinition('test2', 'somename', my_map, language='python')
     >>> view.sync(db)                                       #doctest: +ELLIPSIS
-    [(True, '_design/test2', ...)]
+    [(True, u'_design/test2', ...)]
     >>> design_doc = view.get_doc(db)
     >>> design_doc                                          #doctest: +ELLIPSIS
-    <Document '_design/test2'@'...' {...}>
-    >>> print design_doc['views']['somename']['map']
+    <Document u'_design/test2'@u'...' {...}>
+    >>> print(design_doc['views']['somename']['map'])
     def my_map(doc):
         yield doc['somekey'], doc['somevalue']
     
@@ -113,10 +113,11 @@ class ViewDefinition(object):
         :return: the view results
         :rtype: `ViewResults`
         """
+        wrapper = options.pop('wrapper', self.wrapper)
         merged_options = self.defaults.copy()
         merged_options.update(options)
         return db.view('/'.join([self.design, self.name]),
-                       wrapper=self.wrapper, **merged_options)
+                       wrapper=wrapper, **merged_options)
 
     def __repr__(self):
         return '<%s %r>' % (type(self).__name__, '/'.join([
