@@ -14,23 +14,27 @@ standard library.
 from base64 import b64encode
 from datetime import datetime
 import errno
-from httplib import BadStatusLine, HTTPConnection, HTTPSConnection
 import socket
 import time
-try:
-    from cStringIO import StringIO
-except ImportError:
-    from StringIO import StringIO
 import sys
+
 try:
     from threading import Lock
 except ImportError:
     from dummy_threading import Lock
-import urllib
-from urlparse import urlsplit, urlunsplit
-from email.Utils import parsedate
+
+try:
+    from http.client import BadStatusLine, HTTPConnection, HTTPSConnection
+except ImportError:
+    from httplib import BadStatusLine, HTTPConnection, HTTPSConnection
+
+try:
+    from email.Utils import parsedate
+except ImportError:
+    from email.utils import parsedate
 
 from couchdb import json
+from couchdb import util
 
 __all__ = ['HTTPError', 'PreconditionFailed', 'ResourceNotFound',
            'ResourceConflict', 'ServerError', 'Unauthorized', 'RedirectLimit',
@@ -38,54 +42,6 @@ __all__ = ['HTTPError', 'PreconditionFailed', 'ResourceNotFound',
 __docformat__ = 'restructuredtext en'
 
 
-if sys.version < '2.6':
-
-    class TimeoutMixin:
-        """Helper mixin to add timeout before socket connection"""
-
-        # taken from original python2.5 httplib source code with timeout setting added
-        def connect(self):
-            """Connect to the host and port specified in __init__."""
-            msg = "getaddrinfo returns an empty list"
-            for res in socket.getaddrinfo(self.host, self.port, 0,
-                                          socket.SOCK_STREAM):
-                af, socktype, proto, canonname, sa = res
-                try:
-                    self.sock = socket.socket(af, socktype, proto)
-                    if self.debuglevel > 0:
-                        print "connect: (%s, %s)" % (self.host, self.port)
-
-                    # setting socket timeout
-                    self.sock.settimeout(self.timeout)
-
-                    self.sock.connect(sa)
-                except socket.error, msg:
-                    if self.debuglevel > 0:
-                        print 'connect fail:', (self.host, self.port)
-                    if self.sock:
-                        self.sock.close()
-                    self.sock = None
-                    continue
-                break
-            if not self.sock:
-                raise socket.error, msg
-
-    _HTTPConnection = HTTPConnection
-    _HTTPSConnection = HTTPSConnection
-
-    class HTTPConnection(TimeoutMixin, _HTTPConnection):
-        def __init__(self, *a, **k):
-            timeout = k.pop('timeout', None)
-            _HTTPConnection.__init__(self, *a, **k)
-            self.timeout = timeout
-
-    class HTTPSConnection(TimeoutMixin, _HTTPSConnection):
-        def __init__(self, *a, **k):
-            timeout = k.pop('timeout', None)
-            _HTTPSConnection.__init__(self, *a, **k)
-            self.timeout = timeout
-
-
 if sys.version < '2.7':
 
     from httplib import CannotSendHeader, _CS_REQ_STARTED, _CS_REQ_SENT
@@ -167,9 +123,29 @@ CHUNK_SIZE = 1024 * 8
 
 class ResponseBody(object):
 
-    def __init__(self, resp, callback):
+    def __init__(self, resp, conn_pool, url, conn):
         self.resp = resp
-        self.callback = callback
+        self.chunked = self.resp.msg.get('transfer-encoding') == 'chunked'
+        self.conn_pool = conn_pool
+        self.url = url
+        self.conn = conn
+
+    def __del__(self):
+        if not self.chunked:
+            self.close()
+        else:
+            self.resp.close()
+            if self.conn:
+                # Since chunked responses can be infinite (i.e. for
+                # feed=continuous), and we want to avoid leaking sockets
+                # (even if just to prevent ResourceWarnings when running
+                # the test suite on Python 3), we'll close this connection
+                # eagerly. We can't get it into the clean state required to
+                # put it back into the ConnectionPool (since we don't know
+                # when it ends and we can only do blocking reads). Finding
+                # out whether it might in fact end would be relatively onerous
+                # and require a layering violation.
+                self.conn.close()
 
     def read(self, size=None):
         bytes = self.resp.read(size)
@@ -177,15 +153,18 @@ class ResponseBody(object):
             self.close()
         return bytes
 
+    def _release_conn(self):
+        self.conn_pool.release(self.url, self.conn)
+        self.conn_pool, self.url, self.conn = None, None, None
+
     def close(self):
         while not self.resp.isclosed():
             self.resp.read(CHUNK_SIZE)
-        if self.callback:
-            self.callback()
-            self.callback = None
+        if self.conn:
+            self._release_conn()
 
     def iterchunks(self):
-        assert self.resp.msg.get('transfer-encoding') == 'chunked'
+        assert self.chunked
         while True:
             if self.resp.isclosed():
                 break
@@ -193,7 +172,7 @@ class ResponseBody(object):
             if not chunksz:
                 self.resp.fp.read(2) #crlf
                 self.resp.close()
-                self.callback()
+                self._release_conn()
                 break
             chunk = self.resp.fp.read(chunksz)
             for ln in chunk.splitlines():
@@ -260,14 +239,14 @@ class Session(object):
                 if etag:
                     headers['If-None-Match'] = etag
 
-        if (body is not None and not isinstance(body, basestring) and
+        if (body is not None and not isinstance(body, util.strbase) and
                 not hasattr(body, 'read')):
             body = json.encode(body).encode('utf-8')
             headers.setdefault('Content-Type', 'application/json')
 
         if body is None:
             headers.setdefault('Content-Length', '0')
-        elif isinstance(body, basestring):
+        elif isinstance(body, util.strbase):
             headers.setdefault('Content-Length', str(len(body)))
         else:
             headers['Transfer-Encoding'] = 'chunked'
@@ -276,24 +255,25 @@ class Session(object):
         if authorization:
             headers['Authorization'] = authorization
 
-        path_query = urlunsplit(('', '') + urlsplit(url)[2:4] + ('',))
+        path_query = util.urlunsplit(('', '') + util.urlsplit(url)[2:4] + ('',))
         conn = self.connection_pool.get(url)
 
         def _try_request_with_retries(retries):
             while True:
                 try:
                     return _try_request()
-                except socket.error, e:
+                except socket.error as e:
                     ecode = e.args[0]
                     if ecode not in self.retryable_errors:
                         raise
                     try:
-                        delay = retries.next()
+                        delay = next(retries)
                     except StopIteration:
                         # No more retries, raise last socket error.
                         raise e
-                    time.sleep(delay)
-                    conn.close()
+                    finally:
+                        time.sleep(delay)
+                        conn.close()
 
         def _try_request():
             try:
@@ -303,18 +283,24 @@ class Session(object):
                 if body is None:
                     conn.endheaders()
                 else:
-                    if isinstance(body, str):
-                        conn.endheaders(body)
+                    if isinstance(body, util.strbase):
+                        if isinstance(body, util.utype):
+                            conn.endheaders(body.encode('utf-8'))
+                        else:
+                            conn.endheaders(body)
                     else: # assume a file-like object and send in chunks
                         conn.endheaders()
                         while 1:
                             chunk = body.read(CHUNK_SIZE)
                             if not chunk:
                                 break
-                            conn.send(('%x\r\n' % len(chunk)) + chunk + '\r\n')
-                        conn.send('0\r\n\r\n')
+                            if isinstance(chunk, util.utype):
+                                chunk = chunk.encode('utf-8')
+                            status = ('%x\r\n' % len(chunk)).encode('utf-8')
+                            conn.send(status + chunk + b'\r\n')
+                        conn.send(b'0\r\n\r\n')
                 return conn.getresponse()
-            except BadStatusLine, e:
+            except BadStatusLine as e:
                 # httplib raises a BadStatusLine when it cannot read the status
                 # line saying, "Presumably, the server closed the connection
                 # before sending a valid response."
@@ -333,7 +319,7 @@ class Session(object):
             self.connection_pool.release(url, conn)
             status, msg, data = cached_resp
             if data is not None:
-                data = StringIO(data)
+                data = util.StringIO(data)
             return status, msg, data
         elif cached_resp:
             self.cache.remove(url)
@@ -364,22 +350,21 @@ class Session(object):
             self.connection_pool.release(url, conn)
 
         # Buffer small non-JSON response bodies
-        elif int(resp.getheader('content-length', sys.maxint)) < CHUNK_SIZE:
+        elif int(resp.getheader('content-length', sys.maxsize)) < CHUNK_SIZE:
             data = resp.read()
             self.connection_pool.release(url, conn)
 
         # For large or chunked response bodies, do not buffer the full body,
         # and instead return a minimal file-like object
         else:
-            data = ResponseBody(resp,
-                                lambda: self.connection_pool.release(url, conn))
+            data = ResponseBody(resp, self.connection_pool, url, conn)
             streamed = True
 
         # Handle errors
         if status >= 400:
             ctype = resp.getheader('content-type')
             if data is not None and 'application/json' in ctype:
-                data = json.decode(data)
+                data = json.decode(data.decode('utf-8'))
                 error = data.get('error'), data.get('reason')
             elif method != 'HEAD':
                 error = resp.read()
@@ -402,7 +387,7 @@ class Session(object):
             self.cache.put(url, (status, resp.msg, data))
 
         if not streamed and data is not None:
-            data = StringIO(data)
+            data = util.StringIO(data)
 
         return status, resp.msg, data
 
@@ -431,7 +416,7 @@ class Cache(object):
         self.by_url.pop(url, None)
 
     def _clean(self):
-        ls = sorted(self.by_url.iteritems(), key=cache_sort)
+        ls = sorted(self.by_url.items(), key=cache_sort)
         self.by_url = dict(ls[-self.keep_size:])
 
 
@@ -445,7 +430,7 @@ class ConnectionPool(object):
 
     def get(self, url):
 
-        scheme, host = urlsplit(url, 'http', False)[:2]
+        scheme, host = util.urlsplit(url, 'http', False)[:2]
 
         # Try to reuse an existing connection.
         self.lock.acquire()
@@ -472,7 +457,7 @@ class ConnectionPool(object):
         return conn
 
     def release(self, url, conn):
-        scheme, host = urlsplit(url, 'http', False)[:2]
+        scheme, host = util.urlsplit(url, 'http', False)[:2]
         self.lock.acquire()
         try:
             self.conns.setdefault((scheme, host), []).append(conn)
@@ -488,6 +473,8 @@ class ConnectionPool(object):
 class Resource(object):
 
     def __init__(self, url, session, headers=None):
+        if sys.version_info[0] == 2 and isinstance(url, util.utype):
+            url = url.encode('utf-8') # kind of an ugly hack for issue 235
         self.url, self.credentials = extract_credentials(url)
         if session is None:
             session = Session()
@@ -544,8 +531,8 @@ class Resource(object):
     def _request_json(self, method, path=None, body=None, headers=None, **params):
         status, headers, data = self._request(method, path, body=body,
                                               headers=headers, **params)
-        if 'application/json' in headers.get('content-type'):
-            data = json.decode(data.read())
+        if 'application/json' in headers.get('content-type', ''):
+            data = json.decode(data.read().decode('utf-8'))
         return status, headers, data
 
 
@@ -561,27 +548,34 @@ def extract_credentials(url):
     >>> extract_credentials('http://joe%40example.com:secret@localhost:5984/_config/')
     ('http://localhost:5984/_config/', ('joe@example.com', 'secret'))
     """
-    parts = urlsplit(url)
+    parts = util.urlsplit(url)
     netloc = parts[1]
     if '@' in netloc:
         creds, netloc = netloc.split('@')
-        credentials = tuple(urllib.unquote(i) for i in creds.split(':'))
+        credentials = tuple(util.urlunquote(i) for i in creds.split(':'))
         parts = list(parts)
         parts[1] = netloc
     else:
         credentials = None
-    return urlunsplit(parts), credentials
+    return util.urlunsplit(parts), credentials
 
 
 def basic_auth(credentials):
+    """Generates authorization header value for given credentials.
+    >>> basic_auth(('root', 'relax'))
+    b'Basic cm9vdDpyZWxheA=='
+    >>> basic_auth(None)
+    >>> basic_auth(())
+    """
     if credentials:
-        return 'Basic %s' % b64encode('%s:%s' % credentials)
+        token = b64encode(('%s:%s' % credentials).encode('latin1'))
+        return ('Basic %s' % token.strip().decode('latin1')).encode('ascii')
 
 
 def quote(string, safe=''):
-    if isinstance(string, unicode):
+    if isinstance(string, util.utype):
         string = string.encode('utf-8')
-    return urllib.quote(string, safe)
+    return util.urlquote(string, safe)
 
 
 def urlencode(data):
@@ -589,10 +583,10 @@ def urlencode(data):
         data = data.items()
     params = []
     for name, value in data:
-        if isinstance(value, unicode):
+        if isinstance(value, util.utype):
             value = value.encode('utf-8')
         params.append((name, value))
-    return urllib.urlencode(params)
+    return util.urlencode(params)
 
 
 def urljoin(base, *path, **query):
